package top;

/**
 * the KeyChain does not leak internal objects such as the Keys. Therefore, any Object
 * in the public API is either a Task (depending on the KeyChain implementation, e.g., a Thread or an Interval)
 * or a normal Object (that again has an associated key chain)
 * @author angererc
 *
 */
public final class DummyPermissions<T> extends Permissions<T> {

	
	/*
	 * Exceptions
	 */
	
	public static class KeyChainException extends RuntimeException {
		KeyChainException(String message) {
			super(message);
		}		
	}
	
	
	
	/*
	 * Permissions API
	 */
	
	protected DummyPermissions() {
		System.out.println("!!! Using empty dummy permissions");
	}
	
	@Override
	public T now() {
		return null;
	}
	
	/**
	 * throws an access exception if now() cannot read object o
	 * now can read o if now has been added as a key to o or if somebody
	 * replaced itself with now in o's keychain 
	 * @param o
	 * @throws KeyChainException
	 */
	@Override
	public void checkRead(Object o) throws KeyChainException {
	}
	
	/**
	 * throws AccessException if now() cannot write o
	 * now can write o if it's the only key in o's keychain
	 * @param o
	 * @throws KeyChainException
	 */
	@Override
	public void checkWrite(Object o) throws KeyChainException {
	}

	/**
	 * now must own the slave; then slave's keychain is replaces with the master keychain
	 * the master's key chain must not have been linked to anybody or we throw an exception
	 * @param master
	 * @param slave
	 * @throws KeyChainException
	 */
	@Override
	public void linkKeychains(Object master, Object slave) throws KeyChainException {	
	
	}
	
	/**
	 * now must be owner of slave and slave must have been linked to somebody else before
	 * then slave will get its own keychain again with now as the owner
	 * 
	 */
	@Override
	public void unlinkKeychains(Object slave) {
		
	}
		
	/**
	 * add key to keychain of object o
	 * Throws AccessException if now cannot read o
	 * Lazily creates a single task key for now if no key is associated with o yet
	 * this can lead to unexpected results if o leaks to another task during construction
	 * @param o
	 * @param task
	 * @throws KeyChainException
	 */
	@Override
	public <O> O newObject(O o) throws KeyChainException {	
		
		return o;
	}
	
	/**
	 * add key to keychain of object o
	 * Throws AccessException if now cannot read o
	 * Lazily creates a single task key for now if no key is associated with o yet
	 * this can lead to unexpected results if o leaks to another task during construction
	 * @param o
	 * @param task
	 * @throws KeyChainException
	 */
	@Override
	public void addTask(Object o, T task) throws KeyChainException {	
		
	}
	
	/**
	 * replaces now with key in keychain of object o
	 * Throws AccessException if now cannot read o
	 * Lazily creates a single task key for now if no key is associated with o yet
	 * this can lead to unexpected results if o leaks to another task during construction
	 * @param o
	 * @param task
	 * @throws KeyChainException
	 */
	@Override
	public void replaceNowWithTask(Object o, T task) throws KeyChainException  {		
		
	}
		
	@Override
	public boolean isImmutable(Object o) {
		return false;
	}
	
	@Override
	public void makeImmutable(Object o) throws KeyChainException  {
		
	}
	
	@Override
	public boolean isShared(Object o) {
		return true;
	}
	
	@Override
	public synchronized void makeShared(Object o) throws KeyChainException  {
		
	}
	
}
